$: << File.expand_path('../lib', __FILE__)
require 'rubygems'
require "benchmark"
require 'em-zipper'
require 'net/http'


sites = %w(http://s3.amazonaws.com/ping-em-assets/sample1.txt http://s3.amazonaws.com/ping-em-assets/sample2.txt)
files = %w(sample1.txt sample2.txt).map do |file_name|
  File.join(File.expand_path("./examples/files"), file_name)
end

n     = 10
@count = 0

def em_zipper(collection, n)
  zip  = EM::Zipper.new(collection, StringIO.new).zip!
  zip.callback do |io|
    io.close
    @count = @count + 1
    EM.stop if @count == n
  end
end

def rubyzip(collection, n)
  ::Zip::ZipOutputStream.write_buffer do |zip|
    collection.each do |file_path|
      zip.put_next_entry(File.basename(file_path))

      begin
        if file_path.match(/\Ahttps?:\/\//)
          uri = URI file_path
          Net::HTTP.start(uri.host, uri.port) do |http|
            request = Net::HTTP::Get.new uri

            http.request request do |response|
              response.read_body { |chunk| zip << chunk }
            end
          end
        else
          file = File.open file_path
          while chunk = file.read_nonblock(16384)
            zip << chunk
          end
        end
      rescue EOFError
        # /
      ensure
        file && file.close
      end
    end
  end
end


# //


marks = Benchmark.benchmark do |b|
  b.report("[file] em-zipper") do
    @count = 0

    EM.run do
      n.times do
        EM.next_tick { em_zipper(files, n) }
      end
    end
  end

  b.report('[file] rubyzip') do
    n.times do
      rubyzip(files, n)
    end
  end

  b.report("[net] em-zipper") do
    @count = 0

    EM.run do
      n.times do
        EM.next_tick { em_zipper(sites, n) }
      end
    end
  end

  b.report('[net] rubyzip') do
    n.times do
      rubyzip(sites, n)
    end
  end
end

puts marks

# //
#
# [file] em-zipper  2.780000   0.040000   2.820000 (  2.829317)
# [file] rubyzip    2.690000   0.030000   2.720000 (  2.740578)
# [net]  em-zipper  3.060000   0.150000   3.210000 (  5.076801)
# [net]  rubyzip    3.020000   0.090000   3.110000 (  6.962918)
#   2.780000   0.040000   2.820000 (  2.829317)
#   2.690000   0.030000   2.720000 (  2.740578)
#   3.060000   0.150000   3.210000 (  5.076801)
#   3.020000   0.090000   3.110000 (  6.962918)

