$: << File.expand_path('../lib', __FILE__)
require 'rubygems'
require 'tach'
require 'em-zipper'
require 'net/http'


sites = %w(http://s3.amazonaws.com/ping-em-assets/sample1.txt http://s3.amazonaws.com/ping-em-assets/sample2.txt)
files = %w(sample1.txt sample2.txt).map do |file_name|
  File.join(File.expand_path("./examples/files"), file_name)
end


Tach.meter(10) do 
  tach('[file] em-zipper') do |n|
    @count = 0

    EM.run do
      n.times do |i|
        EM.next_tick { em_zipper files, n }
      end
    end
  end

  tach('[file] rubyzip') do |n|
    n.times { |i| rubyzip sites }
  end

  tach('[net] em-zipper') do |n|
    @count = 0

    EM.run do
      n.times do |i|
        EM.next_tick { em_zipper files, n }
      end
    end
  end

  tach('[net] rubyzip') do |n|
    n.times { |i| rubyzip sites }
  end


  def em_zipper(collection, n)
    zip = EM::Zipper.new(collection, StringIO.new).zip!
    zip.callback do |io|
      io.close
      @count = @count + 1
      EM.stop if @count == n
    end
  end

  def rubyzip(collection)
    ::Zip::ZipOutputStream.write_buffer do |zip|
      collection.each do |file_path|
        zip.put_next_entry(File.basename(file_path))

        begin
          if file_path.match(/\Ahttps?:\/\//)
            uri = URI file_path
            Net::HTTP.start(uri.host, uri.port) do |http|
              request = Net::HTTP::Get.new uri

              http.request request do |response|
                response.read_body { |chunk| zip << chunk }
              end
            end
          else
            file = File.open file_path
            while chunk = file.read_nonblock(16384)
              zip << chunk
            end
          end
        rescue EOFError
          # /
        ensure
          file && file.close
        end
      end
    end
  end
end


# //
#
# File sizes 1.5mb, 503kb
#
# x10
# +------------------+-----------+
# | tach             | total     |
# +------------------+-----------+
# | [net] em-zipper  | 2.727529  |
# +------------------+-----------+
# | [file] em-zipper | 2.753560  |
# +------------------+-----------+
# | [file] rubyzip   | 8.035955  |
# +------------------+-----------+
# | [net] rubyzip    | 10.780907 |
# +------------------+-----------+
#
# x100
# +------------------+-----------+
# | tach             | total     |
# +------------------+-----------+
# | [file] em-zipper | 27.435623 |
# +------------------+-----------+
# | [net] em-zipper  | 27.858313 |
# +------------------+-----------+
# | [net] rubyzip    | 58.306401 |
# +------------------+-----------+
# | [file] rubyzip   | 60.937767 |
# +------------------+-----------+
