require 'em-http-request'

module EventMachine
  module Zipper
    class Base
      include EventMachine::Deferrable

      attr_reader :files, :zos, :fqueue

      def initialize(files, zos)
        @files  = files
        @zos    = zos
        @fqueue = EM::Queue.new
        @count  = 0
      end

      def zip!
        files.each do |file_path|
          EM.next_tick { chunk_data(file_path) }
        end
        fqueue.pop(&write_entry) # begin queue, first file to chunk will be issued this call

        self # ...new(..., ...).zip!
      end


      private

      def close_zos
        zos.close if zos.io.is_a?(File) # close the io if it's a file
        succeed zos.close_buffer        # return the zos
      end

      def write_entry
        proc do |args|
          file_path = args[0]
          q         = args[1]

          entry = Entry.new(file_path, q, zos).write!
          entry.callback do
            @count = @count + 1

            if files.size == @count
              close_zos
            else
              EM.next_tick { fqueue.pop(&write_entry) }
            end
          end
        end
      end

      def chunk_data(file_path)
        chunks = Chunks.new(file_path).chop!
        chunks.callback do |chunk_q|
          EM.next_tick { fqueue.push [file_path, chunk_q] }
        end
      end
    end
  end
end
